#!/usr/bin/perl

use strict;
use warnings;
use File::Basename;

my $ref_type = shift;
my $src_dir = shift;
my $annotation_F = shift;
my $ref_dir = shift;
my $alignment_F = shift;
my $assigned_F = shift;
my $out_dir = shift;

my $alignment_base = basename($alignment_F,".sam");
my $cov_tmp = "$out_dir/coverage_tmp";
my %protein;
my %ann_protein;
my %all_cover;

print " - $assigned_F\n";
`mkdir -p $cov_tmp`;
`samtools view -Sb $alignment_F > $out_dir/$alignment_base.bam`;
`samtools sort	$out_dir/$alignment_base.bam $out_dir/$alignment_base.sorted`;
`bedtools genomecov -bga -ibam $out_dir/$alignment_base.sorted.bam -g $ref_dir/$ref_type.genome.txt > $cov_tmp/genomecov.txt`;


if($ref_type eq "CDS")
{
	open(F,"$annotation_F");
	while(<F>)
	{
		chomp;
   		my ($id,$pos,$str,$syn,$protein_id,$GO_id,$product) = split(/\t/);
    	$ann_protein{$id}{$pos}{$str} = $protein_id;
	}
	close(F);
}

open(F,"$cov_tmp/genomecov.txt");
while(<F>)
{
	chomp;
	my @arr = split(/\t/);
	my $protein_id = "";	
	if($ref_type eq "CDS")
    {
        my $pos;
        my $pos_1;
        my $pos_2;
        my $str;

        my @arr2 = split(/\|/,$arr[0]);
        my $id = $arr2[3];
        if($arr[0] =~ /\|:(\d+)-(\d+)/)
        {
            $str = "+";
            $pos_1 = $1;
            $pos_2 = $2;
        }
        elsif($arr[0] =~ /\|:c(\d+)-(\d+)/)
        {
            $str = "-";
            $pos_1 = $2;
            $pos_2 = $1;
        }

        $pos = "$pos_1-$pos_2";
		
		if(!exists $ann_protein{$id}{$pos}{$str})
		{
			next;
		}

        $protein_id = $ann_protein{$id}{$pos}{$str};
    }
    elsif($ref_type eq "Protein")
    {
        my @arr2 = split(/\|/,$arr[0]);
        $protein_id = $arr2[3];

    }	

    if(!exists $protein{$protein_id})
    {
        $protein{$protein_id} = $arr[3];
    }
    else
    {
        if($protein{$protein_id} != 0)
        {
            $protein{$protein_id} = $arr[3];
        }
    }	
}
close(F);

foreach my $id (keys %protein)
{
    if($protein{$id} != 0)
    {
		$all_cover{$id} = 0;
    }
}

my $base = basename("$assigned_F",".rpkm.txt");
open(W,">$out_dir/$base.filtered.cov.txt");
open(F,"$assigned_F");
while(<F>)
{
	chomp;
	if($_ =~ /^@/)
	{
		print "$_\n";
		next;
	}
	my @arr = split(/\t/);
	if(exists $all_cover{$arr[0]})
	{
		print W "$_\n";
	}
}
close(F);
close(W);

`rm -rf $cov_tmp`;
