#!/usr/bin/perl

use strict;
use warnings;

my $assigned_protein = shift;
my $go_pool_F =shift;
my $annotation_F = shift;
my $qvalue_cutoff = shift;
my $src_dir = shift;
my $out_dir = shift;
my %sample_pro;
my %sample_rpkm;
my %pool;
my %go_tested;
my %go_rpkm;
my %all_protein;

print " - $assigned_protein\n";
my $all_protein_num = `wc -l $annotation_F|awk '{print \$1}'`;
chomp($all_protein_num);

open(F,"$assigned_protein");
while(<F>)
{
	chomp;
	my @arr = split(/\t/);
	my $id = $arr[0];
	$sample_pro{$id} = 0;
	$sample_rpkm{$id} = $arr[5];
}
close(F);
my @sample_P = keys(%sample_pro);
my $sample_num = $#sample_P + 1;

open(F,"$go_pool_F");
while(<F>)
{
	chomp;
	my @arr = split(/\t/);
	my $go = $arr[0];
	my $protein = $arr[1];
	my @arr_protein = split(/;/,$protein);

	foreach my $p (@arr_protein)
	{
		if($p eq ""){next;}
		$pool{$go}{$p} = 0;
	}
}
close(F);

my $hypergeo_R = "$out_dir/hypergeo.R";
open(W1,">$hypergeo_R");
print W1 "#!/usr/bin/env Rscript\n";
foreach my $pool_g (sort{$a cmp $b} keys %pool)
{
	my @pool_P = keys(%{$pool{$pool_g}});
	my $pool_num = $#pool_P+1;
	my $overlap_num = 0;
	foreach my $pool_p (@pool_P)
	{
		if(exists $sample_pro{$pool_p})
		{
			$overlap_num++;
			if(exists $go_rpkm{$pool_g})
			{
				$go_rpkm{$pool_g} += $sample_rpkm{$pool_p};
			}
			else
			{
				$go_rpkm{$pool_g} = $sample_rpkm{$pool_p};
			}
		}
	}
	print W1 "pvalue <- 1-phyper(",$overlap_num-1,",",$sample_num,",",$all_protein_num-$sample_num,",",$pool_num,");\n";
	print W1 "cat(\"$pool_g\\t\", pvalue, \"\\n\")\n";
}
close(W1);
`Rscript $hypergeo_R |sed 's/ //g' > $out_dir/hypergeo_result.txt`;

open(F,"$out_dir/hypergeo_result.txt");
while(<F>)
{
	chomp;
	my @arr = split(/\t/);
	$go_tested{$arr[1]}{$arr[0]} = 0;
}
close(F);

open(W,">$out_dir/go_tested.txt");
foreach my $pvalue (sort{$a <=> $b} keys %go_tested)
{
	foreach my $g (sort{$a cmp $b} keys %{$go_tested{$pvalue}})
	{
		print W "$g\t$pvalue\n";
	}
}
close(W);

`awk '{print \$2}' $out_dir/go_tested.txt > $out_dir/pvalue_tmp.txt`;
`$src_dir/qvalue.R $out_dir/pvalue_tmp.txt $out_dir/qvalue_out.txt`;
`sed '1,3d' $out_dir/qvalue_out.txt | awk \'{print \$2}\' > $out_dir/qvalue.txt`;
`paste $out_dir/go_tested.txt $out_dir/qvalue.txt > $out_dir/go_tested_qval.txt`;

my %q_sort;
open(F,"$out_dir/go_tested_qval.txt");
while(<F>)
{
	chomp;
	my @arr = split(/\t/);
	if($arr[2] == 0 || $arr[2] == 1 || $arr[2] > $qvalue_cutoff)
	{
		next;
	}
	
	$q_sort{$arr[2]}{$arr[0]} = $arr[1];

}
close(F);

open(W,">$out_dir/significant_go.txt");
foreach my $qval (sort{$a<=>$b} keys %q_sort)
{
	foreach my $go (sort{$a cmp $b} keys %{$q_sort{$qval}})
	{
		print W "$go\t$q_sort{$qval}{$go}\t$qval\t$go_rpkm{$go}\n";
	}
}
close(W);

`rm -f $out_dir/hypergeo.R`;
`rm -f $out_dir/hypergeo_result.txt`;
`rm -f $out_dir/go_tested.txt`;
`rm -f $out_dir/qvalue.txt`;
`rm -f $out_dir/pvalue_tmp.txt`;
`rm -f $out_dir/qvalue_out.txt`;
`rm -f $out_dir/go_tested_qval.txt`;
