#!/usr/bin/perl

use strict;
use warnings;

my $idmapping_F = shift;
my $gff_F = shift;
my $obo_F = shift;
my $out_dir = shift;
my %idmap;
my %db_protein;
my %go_pool;
my %obo_go;

system("mkdir -p $out_dir");
system("gunzip -c $idmapping_F | awk 'BEGIN{FS=\"\t\"};{if(\$4!=\"\" && \$7 !=\"\") print \$4FS\$7}' | sed \'s/ //g\' > $out_dir/idmapping_tmp");

open(F,"$out_dir/idmapping_tmp");
while(<F>)
{
	chomp;
	my @arr = split(/\t/);
	my @arr2 = split(/;/,$arr[0]);

	foreach my $p (@arr2)
	{
		$idmap{$p} = $arr[1];
	}
}
close(F);

`mkdir -p $out_dir/gff_tmp`;
`tar xvf $gff_F -C $out_dir/gff_tmp`;
my @gff = `find $out_dir/gff_tmp/|grep -P \"gff\$\"`;
open(W,">$out_dir/fcmm_annotation.txt");
foreach my $gff_F (@gff)
{
	my $locus_tag = "";
	chomp($gff_F);
	open(F,"$gff_F");
	while(<F>)
	{
		chomp;
		if($_ =~ /^#/)
		{
			next;
		}

		my @arr = split(/\t/);
		if($arr[2] eq "gene")
		{
			if($_ =~ /locus_tag=(\S+)/)
			{
				$locus_tag = $1;
			}
		}
		elsif($arr[2] eq "CDS")
		{
			my $id = $arr[0];
			my $start = $arr[3];
			my $end = $arr[4];
			my $str = $arr[6];
			my $pro_id = "";
			my $product = "";
			

			if($_ =~ /protein_id=(\S+);/)
			{
				$pro_id = $1;
				$db_protein{$pro_id} = 0;
			}

			if($_ =~ /product=([^;]+);/)
			{
				$product = $1;
			}
			
			if(!exists $idmap{$pro_id})
			{
				next;
			}

			print W "$id\t$start-$end\t$str\t$locus_tag\t$pro_id\t$idmap{$pro_id}\t$product\n";
		}
	}
	close(F);
}
close(W);

foreach my $k1 (keys %idmap)
{
	if(!exists $db_protein{$k1})
	{
		next;
	}
	my @gos = split(/;/,$idmap{$k1});
	foreach my $g (@gos)
	{
		$go_pool{$g}{$k1} = 0;	
	}
}

open(W2,">$out_dir/go_pool.txt");
foreach my $g (sort{$a cmp $b} keys %go_pool)
{
	print W2 "$g\t";
	foreach my $p (keys %{$go_pool{$g}})
	{
		if(!exists $db_protein{$p})
		{
			next;
		}

		print W2 "$p;";
	}
	print W2 "\n";
}
close(W2);

open(F,"$obo_F");
while(<F>)
{
    chomp;
    if($_ =~ /\[Term\]/)
    {
        my $sline = <F>;
        my $tline = <F>;
        my $fline = <F>;
        chomp($sline);
        chomp($tline);
        chomp($fline);

        my @ids  = ();
        my $function = "";
        my $PCF = "";
        if($sline =~ /^id:\s+(\S+)/)
        {
            push(@ids,$1);
        }

        if($tline =~ /^name:\s+(.+)$/)
        {
            $function = $1;
        }

        if($fline =~ /^namespace:\s+(\S+)/)
        {
            $PCF = $1;
            if($PCF eq "biological_process")
            {
                $PCF = "P";
            }
            elsif($PCF eq "molecular_function")
            {
                $PCF = "F";
            }
            else
            {
                $PCF = "C";
            }
        }

        while(1)
        {
            my $lline = <F>;
            chomp($lline);
            if($lline =~ /^alt_id:\s+(\S+)/)
            {
                push(@ids,$1);
            }
            else
            {
                last;
            }
        }

        foreach my $id (@ids)
        {
            $obo_go{$id} = "$PCF:$function";
        }
    }
}
close(F);

open(W1,">$out_dir/Go2Function.txt");
foreach my $g (sort{$a cmp $b} keys %obo_go)
{
    print W1 "$g\t$obo_go{$g}\n";
}
close(W1);

`rm -f $out_dir/idmapping_tmp`;
`rm -rf $out_dir/gff_tmp`;
